==> hermes-2.4.12.1.tar.gz: hermes-2.4.12.1/lib/hermes/rex_client/tasks/update_contact_task.rb <==
module Hermes
  module RexClient
    class UpdateContactTask

      def update_contact(rex_entity, client = Hermes::RexClient::Client.new, yopmail = false, skip_update_groups = false, dont_update_lessor = false, dont_update_entity = false)
        fields = rex_entity['fields']
        
        possible_roles = []
        possible_roles.push('rdsl') unless dont_update_lessor
        possible_roles.push('cre') unless dont_update_entity

        return unless fields['email'] && fields['id_cristal']  && possible_roles.include?(fields['rolecrelocal'].downcase)

        client.log_puts "updating #{fields['email']} #{fields}"

        if Rails.env.test? || yopmail
          fields['email'] = fields['email'].split('@')[0] + '@yopmail.com'
        end

        contact = User.find_by(id_cristal: fields['id_cristal']) unless fields['id_cristal'].blank?
        contact = User.find_by(email: fields['email'].downcase) unless contact
        
        if contact && contact.email.downcase != fields['email'].downcase
          contact.update(group_ids: [], id_cristal: nil)
          contact = nil
        end
        
        if !contact
          line = 'create new user'
          contact = User.create(user_fields(fields).merge(password: Devise.friendly_token[0,20]))
        else
          line = 'update existing user'
          contact.update(user_fields(fields, contact.login, contact.auth_roles))
        end

        client.log_puts(line)

        contact.create_user_on_identity_provider
        
        unless skip_update_groups
          contact.group_ids = [] unless contact.group_ids

          check_on_params = []
          check_on_params.push({fieldName: 'id_etablissement_cristal', classname: Hermes::Entity }) unless dont_update_entity
          check_on_params.push({fieldName: 'id_bailleur_cristal', classname: Hermes::Lessor }) unless dont_update_lessor

          check_on_params.each do |type|
            query = fields[type[:fieldName]] if fields[type[:fieldName]].is_a? String
            query = {'$in': fields[type[:fieldName]]} if fields[type[:fieldName]].is_a? Array

            ids = type[:classname].where(id_cristal: query).pluck(:manager_group_id) 

            client.log_puts "updating on #{ids.length} ids"
            
            contact.group_ids = (contact.group_ids + (ids || [])).uniq
          end
        end
        
        contact.save!
        contact
      end

      def user_fields(fields, contact_login = nil, auth_roles = [])
        mobileNumber = fields['tel_pro'].blank? ? '1234567890' : fields['tel_pro']
        role = fields['rolecrelocal'].blank? ? 'cre' : fields['rolecrelocal'].downcase
        login = contact_login || fields['email'].downcase
        {
          username: login,
          first_name: fields['prenom'],
          last_name: fields['nom'],
          id_cristal: fields['id_cristal'],
          email: fields['email'].downcase,
          login: login,
          auth_roles: (auth_roles + ['collaborateur offre logement']+ [role]).uniq,
          phone_number: mobileNumber,
          provider: HERMES_CONFIG['identity_provider_key'],
          create_user_data: {
            gender: 'M',
            firstName: fields['prenom'],
            lastName: fields['nom'],
            email: fields['email'].downcase,
            login: login,
            mobileNumber: mobileNumber,
            roles: [role]
          },
        }
      end
    end
  end
end

==> hermes-2.5.1.22.tar.gz: hermes-2.5.1.22/lib/hermes/rex_client/tasks/update_contact_task.rb <==
module Hermes
  module RexClient
    class UpdateContactTask

      def update_contact(rex_entity, client = Hermes::RexClient::Client.new, yopmail = false, skip_update_groups = false, dont_update_lessor = false, dont_update_entity = false)
        fields = rex_entity['fields']
        
        possible_roles = []
        possible_roles.push('rdsl') unless dont_update_lessor
        possible_roles.push('cre') unless dont_update_entity

        return unless fields['email'] && fields['id_cristal']  && possible_roles.include?(fields['rolecrelocal'].downcase)

        client.log_puts "updating #{fields['email']} #{fields}"

        if Rails.env.test? || yopmail
          fields['email'] = fields['email'].split('@')[0] + '@yopmail.com'
        end

        contact = User.find_by(id_cristal: fields['id_cristal']) unless fields['id_cristal'].blank?
        contact = User.find_by(email: fields['email'].downcase) unless contact
        
        if contact && contact.email.downcase != fields['email'].downcase
          contact.update(group_ids: [], id_cristal: nil)
          contact = nil
        end
        
        if !contact
          line = 'create new user'
          contact = User.create(user_fields(fields).merge(password: Devise.friendly_token[0,20]))
        else
          line = 'update existing user'
          contact.update(user_fields(fields, contact.login, contact.auth_roles))
        end

        client.log_puts(line)

        contact.create_user_on_identity_provider
        
        unless skip_update_groups
          contact.group_ids = [] unless contact.group_ids

          check_on_params = []
          check_on_params.push({fieldName: 'id_etablissement_cristal', classname: Hermes::Entity }) unless dont_update_entity
          check_on_params.push({fieldName: 'id_bailleur_cristal', classname: Hermes::Lessor }) unless dont_update_lessor

          check_on_params.each do |type|
            query = fields[type[:fieldName]] if fields[type[:fieldName]].is_a? String
            query = {'$in': fields[type[:fieldName]]} if fields[type[:fieldName]].is_a? Array

            ids = type[:classname].where(id_cristal: query).pluck(:manager_group_id) 

            client.log_puts "updating on #{ids.length} ids"
            
            contact.group_ids = (contact.group_ids + (ids || [])).uniq
          end
        end
        
        contact.save!
        contact
      end

      def user_fields(fields, contact_login = nil, prev_auth_roles = [])
        mobileNumber = fields['tel_pro'].blank? ? '1234567890' : fields['tel_pro']
        role = fields['rolecrelocal'].blank? ? 'cre' : fields['rolecrelocal'].downcase
        auth_roles = (prev_auth_roles + ['collaborateur offre logement'] + [role]).uniq
        login = contact_login || fields['email'].downcase
        {
          username: login,
          first_name: fields['prenom'],
          last_name: fields['nom'],
          id_cristal: fields['id_cristal'],
          email: fields['email'].downcase,
          login: login,
          auth_roles: auth_roles ,
          phone_number: mobileNumber,
          provider: HERMES_CONFIG['identity_provider_key'],
          create_user_data: {
            gender: 'M',
            firstName: fields['prenom'],
            lastName: fields['nom'],
            email: fields['email'].downcase,
            login: login,
            mobileNumber: mobileNumber,
            roles: auth_roles
          },
        }
      end
    end
  end
end
